var createError = require("http-errors");
var express = require("express");
var path = require("path");
var cookieParser = require("cookie-parser");


var logger = require("morgan");

var indexRouter = require("./routes/index");

var app = express();

// view engine setup
app.set("views", path.join(__dirname, "views"));
app.set("view engine", "ejs");

// Setup loggin
if (process.env.NODE_ENV === "development") {
  app.use(logger("dev"));
} else {
  // Do not log successful requests in production
  app.use(logger("dev", {
    skip: (req, res) => { return res.statusCode < 400 }
  }));
}
app.use(express.json());
app.use(express.urlencoded({ extended: false, limit: "1mb" }));
app.use(cookieParser());

app.use((req, res, next) => {
  res.locals.baseDir = "";
  let subPath = req.url.match(/^((\/.*)?\/keys)/);
  let allowed_hosts = [
    "localhost",
    "metager.de",
    "metager.org"
  ];
  if (subPath && "x-forwarded-proto" in req.headers && "x-forwarded-host" in req.headers && "x-forwarded-port" in req.headers) {
    if (allowed_hosts.includes(req.hostname) || (process.env.NODE_ENV === "development" && req.hostname.match(/^(localhost|.*\.ngrok\.io|.*\.review\.metager\.de)$/))) {
      res.locals.baseDir = req.headers["x-forwarded-proto"] + "://" + req.headers["x-forwarded-host"];
      if (!["80", "443"].includes(req.headers["x-forwarded-port"])) {
        res.locals.baseDir += ":" + req.headers["x-forwarded-port"];
      }
      res.locals.baseDir += subPath[1];
    }
  }
  next();
});
app.use(/^((\/.*)?\/keys)/, indexRouter);
app.use(indexRouter);

// catch 404 and forward to error handler
app.use(function (req, res, next) {
  next(createError(404));
});

// error handler
app.use(function (err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = err;

  console.error(err.stack);

  // render the error page
  res.status(err.status || 500);
  res.render("error");
});

module.exports = app;
