const RedisClient = require("./RedisClient");

class Key {
  static get DATABASE_PREFIX() {
    return "key:";
  }

  static get EXPIRATION_AFTER_CHARGE_DAYS() {
    return 365;
  }

  static get KEY_CHARSET() {
    return require("config").get("keys.charset");
  }

  static get KEY_LENGTH() {
    return require("config").get("keys.length");
  }

  static async EXISTS(key) {
    return new Promise((resolve, reject) => {
      let redis_client = RedisClient.CLIENT();
      redis_client.exists(Key.DATABASE_PREFIX + key).then((result) => {
        console.log(result);
        resolve(result);
      });
    });
  }

  /**
   *
   * @returns A new MetaGer-Pass Key with 0 searches
   */
  static async GET_NEW_KEY() {
    return new Promise(async (resolve, reject) => {
      let redis_client = RedisClient.CLIENT();

      do {
        let crypto = require("crypto");
        let key = crypto.randomUUID();

        let key_exists = await redis_client.exists(Key.DATABASE_PREFIX + key);
        if (!key_exists) {
          return resolve(key);
        }
        await new Promise((resolve) => setTimeout(resolve, 50));
      } while (true);
    });
  }

  static async GET_KEY_CHARGE(key) {
    let redis_client = RedisClient.CLIENT();
    return redis_client.get(Key.DATABASE_PREFIX + key).then((key_charge) => {
      return key_charge | 0;
    });
  }

  static async GET_EXPIRATION(key) {
    let dayjs = require("dayjs");
    let redis_client = RedisClient.CLIENT();
    return redis_client
      .expiretime(Key.DATABASE_PREFIX + key)
      .then((key_ttl) => {
        if (key_ttl <= 0) {
          key_ttl = dayjs().add(Key.EXPIRATION_AFTER_CHARGE_DAYS, "day");
        } else {
          key_ttl = dayjs.unix(key_ttl);
        }
        return key_ttl;
      });
  }

  static async CHARGE_KEY(key, amount) {
    let redis_client = RedisClient.CLIENT();
    let expiration = require("dayjs")().add(
      Key.EXPIRATION_AFTER_CHARGE_DAYS,
      "day"
    );
    // Check if key exists and is eligable for recharge
    return redis_client
      .pipeline()
      .incrby(Key.DATABASE_PREFIX + key, amount)
      .expireat(Key.DATABASE_PREFIX + key, expiration.unix())
      .exec();
  }

  static async DISCHARGE_KEY(key, amount) {
    let redis_client = RedisClient.CLIENT();

    // Dicharge key and make sure it can't fall below 0
    return redis_client
      .decrby(Key.DATABASE_PREFIX + key, amount)
      .then((new_charge) => {
        if (new_charge < 0) {
          return redis_client
            .incrby(Key.DATABASE_PREFIX + key, Math.abs(new_charge))
            .then(() => 0);
        } else {
          return new_charge;
        }
      });
  }
}

module.exports = Key;
