var express = require("express");
const Order = require("../app/Order.js");
var router = express.Router({ mergeParams: true });

var lessMiddleware = require("less-middleware");
var browserify = require("browserify-middleware");
var paypalCheckoutRouter = require("../routes/checkout/paypal.js");
var path = require("path");

router.use(lessMiddleware(path.join(__dirname, "..", "public")));
router.use(express.static(path.join(__dirname, "..", "public")));

var authenticationRouter = require("../routes/authentication");
router.use("/", authenticationRouter);

/* GET home page. */
router.get("/", function (req, res, next) {
  res.render("index", {
    price_for_100: Order.PRICE_FOR_100,
    price_for_250: Order.PRICE_FOR_250,
  });
});

var keyRouter = require("../routes/key");
router.use("/key", keyRouter);

router.use("/webhooks/paypal", paypalCheckoutRouter);

// Browserified Javascript files
router.get(
  "/js/base.js",
  browserify(path.join(__dirname, "..", "resources", "js", "base.js"))
);
router.get(
  "/js/funding_sources.js",
  browserify(path.join(__dirname, "..", "resources", "js", "funding_sources.js"))
);
router.get(
  "/js/checkout_paypal.js",
  (req, res, next) => {
    res.header({ "Access-Control-Allow-Origin": "https://www.paypal.com" });
    next();
  },
  browserify(path.join(__dirname, "..", "resources", "js", "checkout_paypal.js"))
);
router.get("/js/enter.js", browserify(path.join(__dirname, "..", "resources", "js", "enter.js")));

module.exports = router;
