var express = require("express");
var router = express.Router({ mergeParams: true });

const config = require("config");
const Key = require("../../app/Key");

// Base URL: /key/:key/orders/:order/refund
router.use("/", (req, res, next) => {
  let refund_count = Math.min(
    Math.max(
      req.data.key.charge,
      req.data.order.order.getAmountRefundRequested()
    ),
    req.data.order.order.getAmount()
  );
  req.data.order.refund = {
    count: refund_count,
    amount: parseFloat(
      (
        Math.ceil(
          (refund_count / req.data.order.order.getAmount()) *
          req.data.order.order.getPrice() *
          100
        ) / 100
      ).toFixed(2)
    ),
  };
  if (req.data.admin === true) {
    req.data.order.refund.approval_link = `${res.locals.baseDir}/key/${req.data.key.key
      }/orders/${req.data.order.order.getOrderID()}/refund/process`;
  }
  req.data.css.push(`${res.locals.baseDir}/styles/orders/refund.css`);
  next();
});
router.get("/", (req, res) => {
  res.render("key", req.data);
});
router.post("/", (req, res, next) => {
  req.data.order.refund.message = req.body.message;

  // Refund values as sent back to us by the user.
  // Used only to verify the amount we are refunding corresponds to what we showed the user
  let user_count = parseInt(req.body.count); // Amount of search request discharged from key

  if (req.data.order.refund.count !== user_count) {
    // Something changed abort here
    req.data.order.refund.error = "invalid_data";
    res.render("key", req.data);
  } else if (req.data.order.order.getAmountRefundRequested() > 0) {
    req.data.order.refund.error = "refund_already_requested";
    res.render("key", req.data);
  } else {
    let request_data = {
      moderation: true,
    };
    req.data.order.refund.moderation_url = `${res.locals.baseDir}/key/${req.data.key.key
      }/orders/${req.data.order.order.getOrderID()}/refund?${new URLSearchParams(
        request_data
      ).toString()}`;
    // Render the message
    let ejs = require("ejs"),
      fs = require("fs"),
      template = fs.readFileSync(
        `${__dirname}/../../views/orders/refund_message.ejs`,
        "utf-8"
      );

    let message = ejs.render(template, req.data);

    // No validation errors. Try to create a new Ticket
    fetch(`${config.get("app.osticket.url")}/api/tickets.json`, {
      method: "post",
      headers: {
        "X-API-Key": config.get("app.osticket.api_key"),
        "Content-Type": "application/json",
      },
      body: JSON.stringify({
        alert: true,
        autorespond: false,
        source: "API",
        name: "Metzgermeister Unbekannt",
        email: "no-reply@metager.de",
        subject: `MetaGer Schlüssel: Erstattung (${req.data.order.order.getOrderID()})`,
        message: `data:text/html;charset=utf-8,${message}`,
        topicId: 12, // ToDo change topic for english autoresponder
      }),
    })
      .then(async (response) => {
        if (response.status != 201) {
          console.error(await response.text());
          throw "Fehler beim Erstellen der Benachrichtigung,";
        } else {
          return response;
        }
      })
      .then((response) =>
        req.data.order.order.requestRefund(req.data.order.refund.count)
      )
      .then((new_charge) => {
        req.data.key.charge = new_charge;
        req.data.order.refund.success = true;
        res.render("key", req.data);
      })
      .catch((reason) => {
        console.log(reason);
        req.data.order.refund.error = "send_email";
        res.render("key", req.data);
      });
  }
});

router.post(
  "/process",
  (req, res, next) => {
    if (!req.data.admin) {
      res.locals.error = { status: 401 };
      res.locals.message = "Unauthorized";
      res.render("error");
    } else {
      next();
    }
  },
  (req, res) => {
    if (req.body.action === "approve") {
      req.data.order.order
        .refund()
        .then(() => {
          req.data.order.refund.result = "REFUNDED";
          res.render("key", req.data);
        })
        .catch((reason) => {
          console.debug(reason);
          res.locals.error = { status: 400 };
          res.locals.message = "Error while executing refund";
        });
    } else {
      req.data.order.order.denyRefund().then(() => {
        req.data.order.refund.result = "REFUND_DENIED";
        res.render("key", req.data);
      });
    }
  }
);

module.exports = router;
